package cz.muni.fi.xludvic.camel.example;

import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.TypeConverter;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.processor.aggregate.AggregationStrategy;

import javax.naming.directory.SearchResult;
import java.util.List;
import java.util.Map;

/**
 * Created by jludvice on 25.2.15.
 */
public class MuniRoute extends RouteBuilder {

    private String propSimple(String propName) {
        return "$simple{property." + propName + "}";
    }

    @Override
    public void configure() throws Exception {
        from("direct:muni").id("muni-java-route")
                .setProperty(Headers.MAIL_BODY).body()
                .setProperty("sender").header("sender")
                .setProperty("Subject").header("Subject")

                .log("new message from sender ${header.sender}")
                .process(new Processor() {
                    @Override
                    public void process(Exchange exchange) throws Exception {

                        String sender = exchange.getIn().getHeader("sender", String.class);
                        System.out.println("processing header sender:" + sender);

                        String login = sender.split("@")[0];
                        System.out.println("found login: " + login);
                        exchange.setProperty(Headers.MUNI_LOGIN, login);
                    }
                })

                .setBody().simple("(description=UCO=$simple{property." + Headers.MUNI_LOGIN + "})")

                .enrich("ldap:ldap?base=ou=People,dc=fi,dc=muni,dc=cz", new AggregationStrategy() {
                    @Override
                    public Exchange aggregate(Exchange exchange, Exchange exchange1) {

                        List<SearchResult> found = exchange1.getIn().getBody(List.class);

                        if (found != null && found.size() > 0) {
                            System.out.println("found " + found.size() + " entries in ldap");

                            TypeConverter converter = exchange.getContext().getTypeConverter();
                            Map firstEntry = converter.convertTo(Map.class, exchange, found.get(0));

                            exchange.setProperty(Headers.LDAP_ENTRIES, firstEntry.entrySet());
                        }
                        return exchange;
                    }
                })

                .removeHeaders("*") // django can't handle all headers from e-mail message

                .setHeader(Exchange.HTTP_PATH).simple("/parse/$simple{property." + Headers.MUNI_LOGIN + "}")
                .setHeader(Exchange.HTTP_METHOD).constant("GET")

                .enrich("http4:localhost:9900", new AggregationStrategy() {
                    @Override
                    public Exchange aggregate(Exchange exchange, Exchange exchange1) {
                        String response = exchange1.getIn().getBody(String.class);
                        exchange.getIn().setHeader(Headers.IS_INFO, response);
                        System.out.println(response);
                        return exchange;
                    }
                })

                .log("found in ldap:")
                .log("${property." + Headers.LDAP_ENTRIES + "}")

                .to("mustache:template.mustache")
                .setHeader(Exchange.FILE_NAME).simple("mail_${property.Subject}.html")
                .to("file:files");

    }
}
