## Apache Camel

- oficiální web [camel.apache.org](http://camel.apache.org)
- kniha Camel in Action,  ([Manning Publications](http://www.manning.com/ibsen/))
- kapitola 3 [zmíněné bakalářské práce](https://is.muni.cz/th/374268/fi_b/)


### Osnova
* koncept cesty
* Camel DSL
* transformace přenášených dat
* Camel & Spring
* Camel komponenty
* chybové řízení
* produkční nasazení Camel aplikace
* monitorování



### Koncept cesty

* posloupnost operací nad přenášenými daty

![camel route](data/img/route_sample.png)



### Zápis cesty

Java DSL

```
from("imaps://imap.gmail.com?" +
    "username=LOGIN&password=PASSWORD&" +
    "delete=false&unseen=true")

    .id("gmail-to-file")

    .setHeader("CamelFileName")
        .groovy("resource:classpath:script.groovy")
    .convertBodyTo(String.class)
    .choice()
        .when(header("sender").contains("@mail.muni.cz"))
            .to("language://python:resource:classpath:is_muni.py")
            .to("file:files")
        .otherwise()
            .to("file:files");
```


Spring DSL

```
<camel:route id="gmail-to-file">
    <camel:from uri="imaps://imap.gmail.com?
        username=LOGIN&amp;password=PASSWORD&amp;
        delete=false&amp;unseen=true"/>

    <camel:setHeader headerName="CamelFileName">
        <camel:groovy>
            resource:classpath:script.groovy
        </camel:groovy>
    </camel:setHeader>
    <camel:convertBodyTo type="java.lang.String"/>
    <camel:choice>
        <camel:when>
            <camel:simple>
                ${header.sender} contains '@mail.muni.cz'
            </camel:simple>
            <camel:to uri="language://
                python:resource:classpath:is_muni.py"/>
            <camel:to uri="file:files"/>
        </camel:when>
        <camel:otherwise>
            <camel:to uri="file:files"/>
        </camel:otherwise>
    </camel:choice>
</camel:route>
```



### Camel a Spring framework

```
<bean id="amq"
      class="org.apache.activemq.camel.component.ActiveMQComponent">
        <property name="brokerURL" value="tcp://localhost:61616"/>
        <property name="userName" value="jack"/>
        <property name="password" value="sparrow"/>
</bean>

<!-- send message to ActiveMQ in Camel Route -->
<to uri="amq:queue:TEST.FOO"/>
```

[ActiveMQ Hello World](http://activemq.apache.org/hello-world.html)


Zasílání zpráv z obyčejné třídy ([Bean Binding](http://camel.apache.org/bean-binding.html)).
```java
import org.apache.camel.EndpointInject;
import org.apache.camel.ProducerTemplate;

public class SomeClass {

    @EndpointInject(uri="amq:queue:TEST.FOO")
    ProducerTemplate template;

    public void sendMessage(String message){
        template.sendBody(message);
    }
}
```

Volání Java třídy z camel cesty
```
from("direct:something")
    .bean(SomeClass.class);
```



### Camel komponenty

[camel.apache.org/components](http://camel.apache.org/components.html)




### chybové řízení

```
<camel:errorHandler id="deadEH" type="DeadLetterChannel"
                    deadLetterUri="file:problematic">
    <camel:redeliveryPolicy maximumRedeliveries="10"
        logStackTrace="true" logHandled="true"
        delayPattern="1:1000;5:5000;10:20000"
        retryAttemptedLogLevel="WARN"
        retriesExhaustedLogLevel="ERROR"/>
</camel:errorHandler>

<onException>
   <exception>java.io.IOException</exception>
   <handled><constant>true</constant></handled>
</onException>
```


<img src="data/img/routes.png" style="width: 50%"/>



### Produkční nasazení

* spustitelná aplikace
    * java -jar app.jar
    * Spring Boot
* webový kontejner / aplikační server
    * JBoss Wildfly
    * Apache Tomcat
* OSGI kontejner
    * JBoss Fuse
    * Apache Karaf



### Monitorování

* JConsole
* Webová konzole [Hawt.io](http://hawt.io/)

![hawtio](data/img/route-summary.png)




# Demo



# ?




Josef Ludvíček

[is.muni.cz/auth/osoba/ludvicek](https://is.muni.cz/auth/osoba/ludvicek)

[ludvicek@mail.muni.cz](mailto:ludvicek@mail.muni.cz)